/* c-lcs.c
 _______________________________________________________________________
|									|
|			      Projet LCS				|
|			  Groupe OLC, CNRS/LAAS				|
|			 7, Av. du Colonel Roche			|
|			   31077 Toulouse Cedex				|
|_______________________________________________________________________|

appel de lcs.

Version 3.4; June 94;

*/



#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#ifdef Solaris
#include <sys/systeminfo.h>
#endif Solaris



/* default values of parameters. -------------------------------------- */

/* parameters: */
#define WSIZE           1024    /* static nominal heap size (kb)        */
#define WFZEN           2.0     /* dynamic nominal heap size (% of old) */
#define WWIDTH          1.2     /* width of free interval in heap       */
#define WTHRES          3.0     /* minor/major gc threshold             */
#define VERBOSE         1       /* verbosity: quiet(0) to verbose(2)    */
#define TSLICE		500     /* half time slice (useless if -none)   */
#define PROTECT		0	/* protection level			*/

/* installation directories and lcsd server info: */
#ifndef EXEC_DIR        /* in case not passed by Makefile ...           */
#define EXEC_DIR	"/usr/local/bin/"        /* where are binaries  */
#define APPS_DIR	"/usr/local/share/lcs/"  /* where are cores     */
#define LIBS_DIR	"/usr/local/lib/lcs/"    /* where are libraries */
#define SERVER          ""                       /* host running lcsd   */
#define PORTNO          4500                     /* informally allocated*/
#endif EXEC_DIR



/* main. --------------------------------------------------------------	*/

/* where args are saved: */
static char lcsbin[80];
static char lcscore[80];
static char wsize[10];
static char wfzen[10];
static char wwidth[10];
static char wthres[10];
static char verbose[2];
static char protect[2];
static char timeslice[10];
static char execdir[80];
static char appsdir[80];
static char libsdir[80];
static char arglist[512];
static char server[64];
static char portno[10];

/* macros: */
#define INFINITY        2147483646   /* a very large int ... */


static int get_host_name (char *host, int len)
{
  /* host gets full hostname (e.g. tippi.laas.fr) */
  char net[256];
						/* e.g.			*/
#ifdef Solaris
  /* Solaris has gethostname, but no getdomainname */
  sysinfo(SI_HOSTNAME,host,len);		/* host			*/
  sysinfo(SI_SRPC_DOMAIN,net,256);		/* site.fr		*/
#else Solaris
  /* Irix 5.X has SI_HOSTNAME, but no SI_SRPC_DOMAIN, Irix 6.X has both */
  /* Linux has none */
  gethostname(host,len);			/* host			*/
  getdomainname(net,256);			/* site.fr		*/
#endif Solaris
  strcat (host,"."); strcat(host,net);		/* host.site.fr		*/
  /* printf ("host= %s\n",host); */
  return 0;
}


void usage (char *s)
{
  fprintf (stderr, s);
  fprintf (stderr,"Usage: lcs [-none|-light|-strong|-file core] [-check] [-help]\n");
  fprintf (stderr,"           [-quiet|-discreet|-verbose] [-memory kb fl] [-gc fl fl]\n");
  fprintf (stderr,"           [-timeslice ms] [-register host port] [-background]\n");
  fprintf (stderr,"           [-protect level] [-arguments a1 ... an|a1 ... an]\n");
}


int Help ()
{
  usage ("");
  printf ("arguments (flags may be abbreviated):\n");
  printf ("FLAGS, ARGS                      NOTES                       DEFAULT\n");
  printf ("-help                            this mode                   \n");
  printf ("-none|-light|-strong|-file core  core to be loaded           -light\n");
  printf ("-check                           just check core, don't run  \n");
  printf ("-quiet|-discreet|-verbose        verbosity level             -discreet\n");
  printf ("-protect int                     security level (>= 0)       -protect 0\n");
  printf ("-memory int fl                   heap sizing parameters      -m %s %s\n",wsize,wfzen);
  printf ("-gc fl fl                        garbage coll. parameters    -g %s %s\n",wwidth,wthres);
  printf ("-timeslice ms                    time slice (useless if -n)  -t %s\n",timeslice);
  printf ("-background                      runs lcs in background      \n");
  printf ("-register host port              lcsd registration service   -r %s %s\n",server,portno);
  printf ("-arguments a1 ... an|a1 ... an   args passed to application  \n");
  exit(0);
}


void wrong ()
{
  usage ("bad command syntax\n"); exit(1);
}


int bad_int (char *str, char *cmd, int lowest, int highest)
{
  long i;
  char *p;
  
  i=strtol(str, &p, 10);
  if ((*p=='\0') && (i>=lowest) && (i<=highest))
    return(0);
  else {
    fprintf(stderr,"bad integer arg for %s\n", cmd);
    return(1);
  }
}


int bad_float (char *str, char *cmd, double lowest)
{
  double f;
  char *p;

  f=strtod(str,&p);
  if ((*p=='\0') && (f>=lowest))
    return(0);
  else {
    fprintf(stderr,"bad float arg for %s\n", cmd);
    return(1);
  }
}


int bad_flag (char *s1, char *s2)  /* 1 iff s1 is not a nonempty initial substring of s2 */
{
  while ((*s1 != '\0') && (*s2 != '\0') && (*s1 == *s2)) {s1++; s2++;}
  return(!((*s1=='\0') || (*s1 == *s2)));
}


void set_core (char *c)
{
  FILE *file;
  char binbuff[10];

  /* on verifie que le core existe, et on de'termine le binaire */

  /* core in working dir ? */
  strcpy(lcscore,c); 
  file = fopen (lcscore,"r");
  if (file==NULL) {
    /* core dans appsdir ? */
    strcpy(lcscore,appsdir); 
    strcat(lcscore,c);
    file = fopen (lcscore,"r");
    if (file==NULL) {
      /* core not found */
      fprintf(stderr,"can't find file (%s)\n",lcscore);
      exit(1);
    }
  }

  /* on e'tablit le binaire, selon label lu dans le core: */
  while ('\n'!=fgetc(file)); /* skip command line */
  while ('\n'!=fgetc(file)); /* skip exec line    */
  fgetc(file);		     /* skips #		  */
  fgets(binbuff,6,file);     /* nom du binaire    */
  binbuff[5]='\0';
  fclose(file);
  if ((0==strcmp(binbuff,"lcs-n")) ||
      (0==strcmp(binbuff,"lcs-l")) || 
      (0==strcmp(binbuff,"lcs-s"))) {
    strcpy(lcsbin,execdir); strcat(lcsbin,"rts"); strcat(lcsbin,&binbuff[3]);
  } else {
    fprintf (stderr, "file (%s) is not an lcs core\n",lcscore);
    exit(1);
  }
}


int main(int argc, char *argv[], char *envp[])
{
  int a=1, over=0;
  char c;
  char *buff;

  /* allow backgrounding: */
  signal (SIGTTIN,SIG_IGN);
  signal (SIGTTOU,SIG_IGN);

  /* set default core, rts, paths, and parameters: */
  sprintf(wsize,"%d",WSIZE);
  sprintf(wfzen,"%.2f",WFZEN);
  sprintf(wwidth,"%.2f",WWIDTH);
  sprintf(wthres,"%.2f",WTHRES);
  sprintf(verbose,"%d", VERBOSE);
  sprintf(protect,"%d", PROTECT);
  sprintf(timeslice,"%d",TSLICE);
  strcpy(execdir,EXEC_DIR);
  strcpy(appsdir,APPS_DIR);
  strcpy(libsdir,LIBS_DIR);
  if (NULL==(buff=(char*)getenv("LCS_SERVER"))) {
    if (strlen(SERVER))
      strcpy(server,SERVER);
    else
      get_host_name(server,64);
  } else
    strcpy(server,buff);
  if (NULL==(buff=(char*)getenv("LCS_PORTNO")))
    sprintf(portno,"%d",PORTNO);
  else
    sprintf(portno,"%d",atoi(buff));
  strcpy(lcscore,appsdir); strcat(lcscore,"lcs-l");
  strcat(lcsbin,execdir); strcat(lcsbin,"rts-l");
  strcpy(arglist,"");

  /* parse command line: */
  while ((!over) && (a < argc) && argv[a][0]=='-') {
    c = argv[a][1];
    switch (c) {
    /* selecting binary {and core}: */
    case 'n' : /* -none */
      if (bad_flag(argv[a],"-none")) wrong();
      set_core("lcs-n");
      a++; 
      break;
    case 'l' : /* -light */
      if (bad_flag(argv[a],"-light")) wrong();
      set_core("lcs-l");
      a++; 
      break;
    case 's' : /* -strong */
      if (bad_flag(argv[a],"-strong")) wrong();
      set_core("lcs-s");
      a++; break;
    case 'f' : /* -file core */
      if (bad_flag(argv[a],"-file") || (a+1 >= argc)) wrong();
      set_core(argv[a+1]);
      a+=2; 
      break;
    case 'c' : /* -check */
      if (bad_flag(argv[a],"-check")) wrong();
      strcpy(lcsbin,execdir); strcat(lcsbin,"rts-c");
      if (-1==execl(lcsbin,lcsbin,lcscore,(char*)0)) {
	perror(lcsbin);
	fprintf (stderr,"bye\n");
	exit(1);
      }
      exit(0);
    case 'h' : /* -help */
      if (bad_flag(argv[a],"-help")) wrong();
      Help();
      break;
    /* parameters: */
    case 'q' : /* -quiet */
      if (bad_flag(argv[a],"-quiet")) wrong();
      strcpy(verbose,"0");
      a+=1;
      break;
    case 'd' : /* -discreet */
      if (bad_flag(argv[a],"-discreet")) wrong();
      strcpy(verbose,"1");
      a+=1;
      break;
    case 'v' : /* -verbose */
      if (bad_flag(argv[a],"-verbose")) wrong();
      strcpy(verbose,"2");
      a+=1;
      break;
    case 'p' : /* -protect lev */
      if (bad_flag(argv[a],"-protect") || (a+1 >= argc) ||
	  bad_int(argv[a+1],argv[a],0,INFINITY)) wrong();
      strcpy(protect,argv[a+1]);
      a+=2; 
      break;
    case 'm' : /* -memory stat, dyn */
      if (bad_flag(argv[a],"-memory") || (a+2 >= argc) ||
	  bad_int(argv[a+1],argv[a],1,INFINITY) ||
	  bad_float(argv[a+2],argv[a],0.0)) wrong();
      strcpy(wsize,argv[a+1]);
      strcpy(wfzen,argv[a+2]);
      a+=3; 
      break;
    case 'g' : /* -gc width, thres */
      if (bad_flag(argv[a],"-gc") || (a+2 >= argc) ||
	  bad_float(argv[a+1],argv[a],1.1) ||
	  bad_float(argv[a+2],argv[a],1.1)) wrong();
      strcpy(wwidth,argv[a+1]);
      strcpy(wthres,argv[a+2]);
      a+=3;
      break;
    case 't' : /* -t timeslice; useless in slcs or lcs -n, but allowed */
      if (bad_flag(argv[a],"-timeslice") || (a+1 >= argc) ||
	  bad_int(argv[a+1],argv[a],0,INFINITY)) wrong();
      strcpy(timeslice,argv[a+1]);
      a+=2;
      break;
    case 'b' : /* -b, background run */
      if (bad_flag(argv[a],"-background")) wrong();
      if (fork()) exit(0);
      a+=1;
      break;
    case 'r' : /* -register host port */
      if (bad_flag(argv[a],"-register") || (a+2 >= argc) ||
	  bad_int(argv[a+2],argv[a],1,INFINITY)) wrong();
      strcpy(server,argv[a+1]);
      if (server[0] == '\0') get_host_name(server,64);
      strcpy(portno,argv[a+2]);
      a+=3;
      break;
    /* for passing args beginning with "-": */
    case 'a' : /* -a ... */
      if (bad_flag(argv[a],"-arguments")) wrong();
      over=1;
      a+=1;
      break;
    default:
      wrong();
      break;}
  }
    
  /* everything left is passed to application */
  if (a < argc) {
    strcpy(arglist,"");
    do {
      strcat(arglist," "); strcat(arglist,argv[a++]);
    } while (a < argc);}

  /* exec: */
  if (-1==execl(lcsbin,lcsbin,lcscore,wsize,wfzen,
		wwidth,wthres,verbose,timeslice,execdir,appsdir,libsdir,"",arglist,server,portno,protect,(char*)0)) {
    perror(lcsbin);
    fprintf (stderr,"bye\n");
    exit(1);
  }

  return(0);
}




